@@ -130,7 +130,7 @@ struct sysfs_attribute *sysfs_open_attribute(const char *path)
 {
 	struct sysfs_attribute *sysattr = NULL;
 	struct stat fileinfo;
-	
+
 	if (path == NULL) {
 		errno = EINVAL;
 		return NULL;
@@ -140,7 +140,7 @@ struct sysfs_attribute *sysfs_open_attribute(const char *path)
 		dprintf("Error allocating attribute at %s\n", path);
 		return NULL;
 	}
-	if (sysfs_get_name_from_path(path, sysattr->name, 
+	if (sysfs_get_name_from_path(path, sysattr->name,
 				SYSFS_NAME_LEN) != 0) {
 		dprintf("Error retrieving attrib name from path: %s\n", path);
 		sysfs_close_attribute(sysattr);
@@ -174,12 +174,12 @@ int sysfs_write_attribute(struct sysfs_attribute *sysattr,
 {
 	int fd;
 	int length;
-	
+
 	if (sysattr == NULL || new_value == NULL || len == 0) {
 		errno = EINVAL;
 		return -1;
 	}
-	
+
 	if (!(sysattr->method & SYSFS_METHOD_STORE)) {
 		dprintf ("Store method not supported for attribute %s\n",
 			sysattr->path);
@@ -188,22 +188,23 @@ int sysfs_write_attribute(struct sysfs_attribute *sysattr,
 	}
 	if (sysattr->method & SYSFS_METHOD_SHOW) {
 		/*
-		 * read attribute again to see if we can get an updated value 
+		 * read attribute again to see if we can get an updated value
 		 */
 		if ((sysfs_read_attribute(sysattr)) != 0) {
 			dprintf("Error reading attribute\n");
 			return -1;
 		}
-		if ((strncmp(sysattr->value, new_value, sysattr->len)) == 0) {
+		if (((strncmp(sysattr->value, new_value, sysattr->len)) == 0)
+				&& (len == sysattr->len)) {
 			dprintf("Attr %s already has the requested value %s\n",
 					sysattr->name, new_value);
-			return 0;	
+			return 0;
 		}
 	}
-	/* 
+	/*
 	 * open O_WRONLY since some attributes have no "read" but only
-	 * "write" permission 
-	 */ 
+	 * "write" permission
+	 */
 	if ((fd = open(sysattr->path, O_WRONLY)) < 0) {
 		dprintf("Error reading attribute %s\n", sysattr->path);
 		return -1;
@@ -216,9 +217,9 @@ int sysfs_write_attribute(struct sysfs_attribute *sysattr,
 		close(fd);
 		return -1;
 	} else if ((unsigned int)length != len) {
-		dprintf("Could not write %d bytes to attribute %s\n", 
+		dprintf("Could not write %d bytes to attribute %s\n",
 					len, sysattr->name);
-		/* 
+		/*
 		 * since we could not write user supplied number of bytes,
 		 * restore the old value if one available
 		 */
@@ -228,12 +229,12 @@ int sysfs_write_attribute(struct sysfs_attribute *sysattr,
 			return -1;
 		}
 	}
-	
+
 	/*
 	 * Validate length that has been copied. Alloc appropriate area
 	 * in sysfs_attribute. Verify first if the attribute supports reading
 	 * (show method). If it does not, do not bother
-	 */ 
+	 */
 	if (sysattr->method & SYSFS_METHOD_SHOW) {
 		if (length != sysattr->len) {
 			sysattr->value = (char *)realloc
@@ -241,12 +242,12 @@ int sysfs_write_attribute(struct sysfs_attribute *sysattr,
 			sysattr->len = length;
 			safestrcpymax(sysattr->value, new_value, length);
 		} else {
-			/*"length" of the new value is same as old one */ 
+			/*"length" of the new value is same as old one */
 			safestrcpymax(sysattr->value, new_value, length);
 		}
 	}
-			
-	close(fd);	
+
+	close(fd);
 	return 0;
 }
 
@@ -292,7 +293,7 @@ int sysfs_read_attribute(struct sysfs_attribute *sysattr)
 		return -1;
 	}
 	if (sysattr->len > 0) {
-		if ((sysattr->len == length) && 
+		if ((sysattr->len == length) &&
 				(!(strncmp(sysattr->value, fbuf, length)))) {
 			close(fd);
 			free(fbuf);
@@ -315,14 +316,14 @@ int sysfs_read_attribute(struct sysfs_attribute *sysattr)
 
 /**
  * sysfs_read_attribute_value: given path to attribute, return its value.
- *	values can be up to a pagesize, if buffer is smaller the value will 
- *	be truncated. 
+ *	values can be up to a pagesize, if buffer is smaller the value will
+ *	be truncated.
  * @attrpath: sysfs path to attribute
  * @value: buffer to put value
  * @vsize: size of value buffer
  * returns 0 with success and -1 with error.
  */
-int sysfs_read_attribute_value(const char *attrpath, 
+int sysfs_read_attribute_value(const char *attrpath,
 					char *value, size_t vsize)
 {
 	struct sysfs_attribute *attr = NULL;
@@ -345,7 +346,7 @@ int sysfs_read_attribute_value(const char *attrpath,
 		return -1;
 	}
 	length = strlen(attr->value);
-	if (length > vsize) 
+	if (length > vsize)
 		dprintf("Value length %d is larger than supplied buffer %d\n",
 			length, vsize);
 	safestrcpymax(value, attr->value, vsize);
@@ -355,16 +356,16 @@ int sysfs_read_attribute_value(const char *attrpath,
 }
 
 /**
- * sysfs_get_value_from_attrbutes: given a linked list of attributes and an 
+ * sysfs_get_value_from_attrbutes: given a linked list of attributes and an
  * 	attribute name, return its value
  * @attr: attribute to search
  * @name: name to look for
  * returns char * value - could be NULL
  */
 char *sysfs_get_value_from_attributes(struct dlist *attr, const char *name)
-{	
+{
 	struct sysfs_attribute *cur = NULL;
-	
+
 	if (attr == NULL || name == NULL) {
 		errno = EINVAL;
 		return NULL;
@@ -382,7 +383,7 @@ char *sysfs_get_value_from_attributes(struct dlist *attr, const char *name)
  */
 void sysfs_close_link(struct sysfs_link *ln)
 {
-	if (ln != NULL) 
+	if (ln != NULL)
 		free(ln);
 }
 
@@ -393,11 +394,11 @@ void sysfs_close_link(struct sysfs_link *ln)
 void sysfs_close_directory(struct sysfs_directory *sysdir)
 {
 	if (sysdir != NULL) {
-		if (sysdir->subdirs != NULL) 
+		if (sysdir->subdirs != NULL)
 			dlist_destroy(sysdir->subdirs);
 		if (sysdir->links != NULL)
 			dlist_destroy(sysdir->links);
-		if (sysdir->attributes != NULL) 
+		if (sysdir->attributes != NULL)
 			dlist_destroy(sysdir->attributes);
 		free(sysdir);
 		sysdir = NULL;
@@ -437,11 +438,11 @@ int sysfs_read_all_subdirs(struct sysfs_directory *sysdir)
 		errno = EINVAL;
 		return -1;
 	}
-	if (sysdir->subdirs == NULL) 
-		if ((sysfs_read_dir_subdirs(sysdir)) != 0) 
+	if (sysdir->subdirs == NULL)
+		if ((sysfs_read_dir_subdirs(sysdir)) != 0)
 			return 0;
 	if (sysdir->subdirs != NULL) {
-		dlist_for_each_data(sysdir->subdirs, cursub, 
+		dlist_for_each_data(sysdir->subdirs, cursub,
 						struct sysfs_directory) {
 			if ((sysfs_read_dir_subdirs(cursub)) != 0) {
 				dprintf ("Error reading subdirectory %s\n",
@@ -513,6 +514,7 @@ struct sysfs_link *sysfs_open_link(const char *linkpath)
 	safestrcpy(ln->path, linkpath);
 	if ((sysfs_get_name_from_path(linkpath, ln->name, SYSFS_NAME_LEN)) != 0
 	    || (sysfs_get_link(linkpath, ln->target, SYSFS_PATH_MAX)) != 0) {
+		sysfs_close_link(ln);
 		errno = EINVAL;
 		dprintf("Invalid link path %s\n", linkpath);
 		return NULL;
@@ -543,7 +545,7 @@ static int add_attribute(struct sysfs_directory *sysdir, const char *path)
 			return 0;
 		}
 	}
-			                        
+
 	if (sysdir->attributes == NULL) {
 		sysdir->attributes = dlist_new_with_delete
 			(sizeof(struct sysfs_attribute), sysfs_del_attribute);
@@ -752,7 +754,7 @@ int sysfs_read_directory(struct sysfs_directory *sysdir)
 			dprintf("stat failed\n");
 			continue;
 		}
-		if (S_ISDIR(astats.st_mode)) 
+		if (S_ISDIR(astats.st_mode))
 			retval = add_subdirectory(sysdir, file_path);
 
 		else if (S_ISLNK(astats.st_mode))
@@ -788,7 +790,7 @@ int sysfs_refresh_dir_attributes(struct sysfs_directory *sysdir)
 		sysdir->attributes = NULL;
 	}
 	if ((sysfs_read_dir_attributes(sysdir)) != 0) {
-		dprintf("Error refreshing attributes for directory %s\n", 
+		dprintf("Error refreshing attributes for directory %s\n",
 							sysdir->path);
 		return 1;
 	}
@@ -817,7 +819,7 @@ int sysfs_refresh_dir_links(struct sysfs_directory *sysdir)
 		sysdir->links = NULL;
 	}
 	if ((sysfs_read_dir_links(sysdir)) != 0) {
-		dprintf("Error refreshing links for directory %s\n", 
+		dprintf("Error refreshing links for directory %s\n",
 							sysdir->path);
 		return 1;
 	}
@@ -846,7 +848,7 @@ int sysfs_refresh_dir_subdirs(struct sysfs_directory *sysdir)
 		sysdir->subdirs = NULL;
 	}
 	if ((sysfs_read_dir_subdirs(sysdir)) != 0) {
-		dprintf("Error refreshing subdirs for directory %s\n", 
+		dprintf("Error refreshing subdirs for directory %s\n",
 							sysdir->path);
 		return 1;
 	}
@@ -862,7 +864,7 @@ int sysfs_refresh_dir_subdirs(struct sysfs_directory *sysdir)
  *
  * NOTE: Since we know the attribute to look for, this routine looks for the
  * 	attribute if it was created _after_ the attrlist was read initially.
- * 	
+ *
  * returns sysfs_attribute if found and NULL if not found
  */
 struct sysfs_attribute *sysfs_get_directory_attribute
@@ -870,21 +872,22 @@ struct sysfs_attribute *sysfs_get_directory_attribute
 {
 	struct sysfs_attribute *attr = NULL;
 	char new_path[SYSFS_PATH_MAX];
-	
+
 	if (dir == NULL || attrname == NULL) {
 		errno = EINVAL;
 		return NULL;
 	}
 
-	if (dir->attributes == NULL) 
-		if ((sysfs_read_dir_attributes(dir) != 0) 
+	if (dir->attributes == NULL)
+		if ((sysfs_read_dir_attributes(dir) != 0)
 		    || (dir->attributes == NULL))
 			return NULL;
 
 	attr = (struct sysfs_attribute *)dlist_find_custom
 			(dir->attributes, attrname, dir_attribute_name_equal);
 	if (attr != NULL) {
-		if ((sysfs_read_attribute(attr)) != 0) {
+		if ((attr->method & SYSFS_METHOD_SHOW) &&
+			(sysfs_read_attribute(attr)) != 0) {
 			dprintf("Error reading attribute %s\n", attr->name);
 			return NULL;
 		}
@@ -901,7 +904,7 @@ struct sysfs_attribute *sysfs_get_directory_attribute
 			}
 		}
 	}
-		
+
 	return attr;
 }
 
@@ -922,7 +925,7 @@ struct sysfs_link *sysfs_get_directory_link
 		if ((sysfs_read_dir_links(dir) != 0) || (dir->links == NULL))
 			return NULL;
 	} else {
-		if ((sysfs_refresh_dir_links(dir)) != 0) 
+		if ((sysfs_refresh_dir_links(dir)) != 0)
 			return NULL;
 	}
 
@@ -952,11 +955,11 @@ struct sysfs_directory *sysfs_get_subdirectory(struct sysfs_directory *dir,
 
 	sub = (struct sysfs_directory *)dlist_find_custom(dir->subdirs,
 		subname, dir_subdir_name_equal);
-	if (sub != NULL) 
+	if (sub != NULL)
 		return sub;
 
 	if (dir->subdirs != NULL) {
-		dlist_for_each_data(dir->subdirs, cursub, 
+		dlist_for_each_data(dir->subdirs, cursub,
 					struct sysfs_directory) {
 			if (cursub->subdirs == NULL) {
 				if (sysfs_read_dir_subdirs(cursub) != 0)
@@ -993,12 +996,12 @@ struct sysfs_link *sysfs_get_subdirectory_link(struct sysfs_directory *dir,
 	if (ln != NULL)
 		return ln;
 
-	if (dir->subdirs == NULL) 
+	if (dir->subdirs == NULL)
 		if (sysfs_read_dir_subdirs(dir) != 0)
 			return NULL;
 
 	if (dir->subdirs != NULL) {
-		dlist_for_each_data(dir->subdirs, cursub, 
+		dlist_for_each_data(dir->subdirs, cursub,
 						struct sysfs_directory) {
 			ln = sysfs_get_subdirectory_link(cursub, linkname);
 			if (ln != NULL)
